@@ -137,7 +137,7 @@ class DictFilter(collections.abc.Mapping):
         return filter(self.include_keys.__contains__, self.dict.keys())
 
     def __len__(self):
-        return len(list(self))
+        return len(self.include_keys & self.dict.keys())
 
 
 def dict_map(function, dictionary):
@@ -279,7 +279,7 @@ class RangeMap(dict):
         return (sorted_keys[RangeMap.first_item], sorted_keys[RangeMap.last_item])
 
     # some special values for the RangeMap
-    undefined_value = type(str('RangeValueUndefined'), (), {})()
+    undefined_value = type('RangeValueUndefined', (), {})()
 
     class Item(int):
         "RangeMap Item"
@@ -307,6 +307,7 @@ def sorted_items(d, key=__identity, reverse=False):
     >>> tuple(sorted_items(sample, reverse=True))
     (('foo', 20), ('baz', 10), ('bar', 42))
     """
+
     # wrap the key func so it operates on the first element of each item
     def pairkey_key(item):
         return key(item[0])
@@ -475,7 +476,7 @@ class ItemsAsAttributes:
     Mix-in class to enable a mapping object to provide items as
     attributes.
 
-    >>> C = type(str('C'), (dict, ItemsAsAttributes), dict())
+    >>> C = type('C', (dict, ItemsAsAttributes), dict())
     >>> i = C()
     >>> i['foo'] = 'bar'
     >>> i.foo
@@ -504,7 +505,7 @@ class ItemsAsAttributes:
 
     >>> missing_func = lambda self, key: 'missing item'
     >>> C = type(
-    ...     str('C'),
+    ...     'C',
     ...     (dict, ItemsAsAttributes),
     ...     dict(__missing__ = missing_func),
     ... )
@@ -579,7 +580,7 @@ class IdentityOverrideMap(dict):
         return key
 
 
-class DictStack(list, collections.abc.Mapping):
+class DictStack(list, collections.abc.MutableMapping):
     """
     A stack of dictionaries that behaves as a view on those dictionaries,
     giving preference to the last.
@@ -596,11 +597,12 @@ class DictStack(list, collections.abc.Mapping):
     >>> stack.push(dict(a=3))
     >>> stack['a']
     3
+    >>> stack['a'] = 4
     >>> set(stack.keys()) == set(['a', 'b', 'c'])
     True
-    >>> set(stack.items()) == set([('a', 3), ('b', 2), ('c', 2)])
+    >>> set(stack.items()) == set([('a', 4), ('b', 2), ('c', 2)])
     True
-    >>> dict(**stack) == dict(stack) == dict(a=3, c=2, b=2)
+    >>> dict(**stack) == dict(stack) == dict(a=4, c=2, b=2)
     True
     >>> d = stack.pop()
     >>> stack['a']
@@ -611,6 +613,9 @@ class DictStack(list, collections.abc.Mapping):
     >>> stack.get('b', None)
     >>> 'c' in stack
     True
+    >>> del stack['c']
+    >>> dict(stack)
+    {'a': 1}
     """
 
     def __iter__(self):
@@ -631,6 +636,18 @@ class DictStack(list, collections.abc.Mapping):
     def __len__(self):
         return len(list(iter(self)))
 
+    def __setitem__(self, key, item):
+        last = list.__getitem__(self, -1)
+        return last.__setitem__(key, item)
+
+    def __delitem__(self, key):
+        last = list.__getitem__(self, -1)
+        return last.__delitem__(key)
+
+    # workaround for mypy confusion
+    def pop(self, *args, **kwargs):
+        return list.pop(self, *args, **kwargs)
+
 
 class BijectiveMap(dict):
     """
